@@ -4,6 +4,10 @@ library("mvtnorm")
 library("colorspace")
 library("latex2exp")
 
+### version 1.0-6 only
+mmlt <- Mmlt
+mmltoptim <- Mmltoptim
+
 set.seed(42)
 par(ask = TRUE)
 
